import os
import json
import base64
import requests
import hashlib
from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5 as Signature_pkcs1_v1_5

"""
function 2:
openssl genrsa -out ./myPrivateKey.pem -passout pass:"密码" -des3 2048
openssl rsa -pubout -in ./myPrivateKey.pem -passin pass:"密码" -out ./myPublicKey.pem
"""


def gen_sig_(unsign_data, secret):
    """
    两种方式生成签名：
		这是第一种
        1. 打开文件方式，见所有被注释的部分，文件为.pem
        2. passphrase 的参数为文件加密的密码
    :return: 签名数据
    """

    path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    doc = os.path.join(path, 'myPrivateKey.pem')
    with open(doc) as pk:
        key_data = pk.read()
    rsaKey = RSA.importKey(key_data, passphrase="密码")
    signer = Signature_pkcs1_v1_5.new(rsaKey)
    digest = SHA256.new()
    digest.update(unsign_data.encode('utf8'))
    sign = signer.sign(digest)
    signature = base64.b64encode(sign)
    print(signature)
    return signature


def verify_sign_(data, sign, secret):
    """
	读取公钥文件，验签
    """
    path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    doc = os.path.join(path, 'myPrivateKey.pem')
    with open(doc) as pk:
        key_data = pk.read()
    rsaKey = RSA.importKey(key_data, passphrase=b'mima')

    verifier = Signature_pkcs1_v1_5.new(rsaKey)
    digest = SHA256.new()
    digest.update(data.encode('utf8'))
    is_verify = verifier.verify(digest, base64.b64decode(sign))
    print(is_verify)
    return is_verify


def gen_sign(private_key, unsign_data, secret):
    """
    两种方式生成签名：
        1. 是直接读取私钥的方式和未加签的数据
    :return: 签名数据
    """
    rsaKey = RSA.importKey(base64.b64decode(private_key), passphrase=secret)
    signer = Signature_pkcs1_v1_5.new(rsaKey)
    digest = SHA256.new()
    digest.update(unsign_data.encode('utf8'))
    sign = signer.sign(digest)
    signature = base64.b64encode(sign)
    return signature.decode()


def verify_sign(pubkey, data, sign, secret):
    """
	直接传公钥方式
    """
    rsaKey = RSA.importKey(base64.b64decode(pubkey), passphrase=secret)
    verifier = Signature_pkcs1_v1_5.new(rsaKey)
    digest = SHA256.new()
    digest.update(data.encode('utf8'))
    is_verify = verifier.verify(digest, base64.b64decode(sign))
    print(is_verify)
    return is_verify


def generate_sign(param):
    '''生成hash'''
    stringA = "&".join([f"{k}={param[k]}" for k in sorted(param)])
    # SHA256加密
    hash_sha256 = hashlib.sha256(stringA.encode('utf8')).hexdigest()
    return hash_sha256


if __name__ == '__main__':
    # 读取私钥
    with open("ty_rsa", mode="r", encoding="utf-8") as f:
        privetkey = "".join(f.readlines()[1:-1])
    privetkey = privetkey.encode(encoding='utf-8')

    # 生成hash
    params = {
        'appid': "fea79d4c-9906-408c-86a5-8a092fda49b3",  # 爱城市网appID
        'code': "c417598b-5892-4217-81be-c3b6080e9fa1",  # 授权成功后产生的code
        'grant_type': "authorization_code"
    }
    hash_value = generate_sign(params)

    # 签名
    sign_value = gen_sign(privetkey, hash_value, None)
    params["sign"] = sign_value

    # 换取token
    resp = requests.post(url="https://auth.icity24.cn/icity/auth2.0/token", data=params)
    resp = json.loads(resp.text)
    print(resp)
    if resp["code"] != 1000:
        print("验证失败")
